package middleware

import (
	"fmt"
	"github.com/kataras/iris"
	"github.com/kataras/iris/context"
	"lapp_-wy/conf"
	"lapp_-wy/utils"
	"lapp_-wy/web/middleware/casbins"
	"lapp_-wy/web/middleware/glog"
	"lapp_-wy/web/middleware/jwts"
	"lapp_-wy/web/supports"
	"runtime"
	"strings"
)

type Middleware struct {
}

func ServeHTTP(ctx context.Context) {
	path := ctx.Path()
	// 过滤静态资源、login接口、首页等...不需要验证
	if checkURL(path) || strings.Contains(path, "/public") {
		ctx.Next()
		return
	}

	// jwt token拦截
	if !jwts.Serve(ctx) {
		return
	}

	// 系统菜单不进行权限拦截
	publicRutes := conf.AppConfig.PublicRute
	res := utils.IsContain(publicRutes, path)
	if !res {
		// casbin权限拦截
		ok := casbins.CheckPermissions(ctx)
		if !ok {
			return
		}
	}

	// Pass to real API
	ctx.Next()
}

/**
return
	true:则跳过不需验证，如登录接口等...
	false:需要进一步验证
*/
func checkURL(reqPath string) bool {
	//过滤
	for _, v := range conf.AppConfig.Other.IgnoreURLs {
		if reqPath == v {
			return true
		}
	}
	return false
}

/**
* @Description: 错误信息处理
* @Author: zhangxin
* @Date: 2021/03/23
 */
func Recover(ctx iris.Context) {
	defer func() {
		if err := recover(); err != nil {
			if ctx.IsStopped() {
				return
			}
			var stacktrace string
			for i := 1; ; i++ {
				_, f, l, got := runtime.Caller(i)
				if !got {
					break
				}
				stacktrace += fmt.Sprintf("%s:%d\n", f, l)
			}
			glog.ErrorExtln("recovery","recovery", stacktrace)

			// 打印错误日志
			supports.Error(ctx, iris.StatusInternalServerError, "操作出错请重试", nil)
			return
		}
	}()
	ctx.Next()
}